import os
import time
import math
import json
import asyncio
from .constants import ADD_DEVICES, PASSED, FAILED
from .logger import logger


def add_device_using_mac_serial(central_api_req, input_data : dict, **kwargs) -> dict:
    """
    Adding device to the Central Account

        Parameters:
            central_api_req (CentralAPIRequest) : Request object to make HTTP call
            input_data (json): Input data(mac and serial) to add devices
            **kwargs: Additional or Optional arguments

        Returns:
            list(dict): includes status_code, input(list(dict)) and ouput(str)
    """
    try:
        input_list = []
        for each_input_dict in input_data:
            if ('mac' and 'serial') in each_input_dict.keys():
                input_list.append(each_input_dict)
            else:
                raise Exception("Invalid request data")
        input_data = input_list
        limit = 20
        add_device_output_list = []
        if len(input_data) > 0:
            for index in range(math.ceil(len(input_data) / limit)):
                subtasks = []
                for device_info in input_data[index * limit:(index + 1) * limit]:
                    subtasks.append(add_device(central_api_req, device_info))
                if os.name == "nt":
                    asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
                add_device_output_list.extend(asyncio.run(synctask(subtasks)))
                if len(input_data[index * limit:(index + 1) * limit]) > 20:
                    logger.debug("Time sleep started")
                    time.sleep(20)
                    logger.debug("Time sleep ended")
            return add_device_output_list
        else:
            error_message = "0 device selected to perform the task"
            logger.error(error_message)
            serial_list = [each_dict_in_input['serial'] for each_dict_in_input in input_data]
            return [
                {"serial": serial_list, "status": FAILED, "message": error_message}
            ]
    except Exception as e:
        logger.error(e)
        error_message = "Error adding the device to central inventory"
        logger.error(error_message)
        serial_list = [each_dict_in_input['serial'] for each_dict_in_input in input_data]
        return [
            {"serial": serial_list, "status": FAILED, "message": error_message}
        ]


async def add_device(central_api_req, device_info):
    """
    Asynchronous function to add device to central

        Parameters:
                central_api_req (CentralAPIRequest) : Request object to make HTTP call
                device_info (dictionary): single row from the input file

        Returns:
            dict: includes status_code, input(list(dict)) and ouput(str)
    """
    try:
        serial = device_info.get('serial')
        method = "POST"
        api_path = ADD_DEVICES
        resp, resp_data = await central_api_req.asyncrequest(method, api_path, data=json.dumps([device_info]))
        if resp.status == 200:
            error_message = f"{serial} - Device added"
            logger.info(f"{resp.status} - Device added")
            return {"serial": serial, "status": PASSED, "message": error_message}
        elif resp.status == 429:
            error_message = f"{resp.status} Central API Error - API Rate limit exceeded"
            logger.error(error_message)
            return {"serial": serial, "status": FAILED, "message": error_message}
            
        elif resp.status == 500:
            error_message = f"{resp.status} Central API Error - Internal server error"
            logger.error(error_message)
            return {"serial": serial, "status": FAILED, "message": error_message}
            
        elif resp.status == 400 and resp_data.get("code") == "ATHENA_ERROR_NO_DEVICE" and resp_data.get("extra").get("message").get("blocked_devices") != []:
            error_message = f"{resp.status} - Devices are blocked, check manually if the devices already added"
            logger.error(error_message)
            return {"serial": serial, "status": FAILED, "message": error_message}
                
        else:
            error_message = f"Failed to add device into central"
            logger.error(error_message)
            return {"serial": serial, "status": FAILED, "message": error_message}
            
    except Exception as e:
        logger.error(e)
        error_message = f"Error in adding AP to inventory"
        logger.error(error_message)
        return {"serial": serial, "status": FAILED, "message": error_message}
        
async def synctask(subtasks):
    task_level_output = await asyncio.gather(*subtasks)
    return task_level_output
